c----------------------------------------------------------------------
c
c                         ====================
c                          subroutine MATRXO
c                         ====================
c
c    This routine calculates the matrix elements off-diagonal subspace
c (pS=+/- 1, qS=0).  Nonsecular terms in the Hamiltonian is ignored
c due to the high field approximation.  This is done by not allowing
c the coupling between the spaces with different pS indices.  The only
c source of the coupling between diagonal and off-diagonal subspaces is
c pulse propagator.
c
c    M-symmetrization is included.  In the absence of nuclear Zeeman,
c the matrix becomes block-diagonal form and we need to keep only the
c block with jM=1 since the starting vector has non-zero elements in
c jM=1 and pulse propagator does not mix jM=1 and jM=-1 block.  In the
c presence of nuclear Zeeman, the block with jM=-1 is included in the
c calculation.
c
c    This routine is based upon EPRLL routines ver.1.3., whose features are
c      1. improved matrix storage algorithm (store only half of the matrix)
c      2. support nonaxial diffusion tensor R (in case of Brownian motion)
c      3. uses pruning scheme by eprbl and tnll.  eprbl and tnll generates
c         an <fileid>.ind file which keeps only the basis elements with
c         significant projection on the starting vector.
c
c    The phenomenological relaxation processes are also included as
c    T2ed, T2ef for the electronic relaxation.
c
c      T2ed : orientationally invariant electronic T2 relaxation
c             This defines the homogeneous line-width of ESR transition.
c      T2ef : angular variation of T2e values.  This angular variation
c             is naturally defined by the angle between B0 and the
c             molecular z-axis.  It is defined in the angle between
c             director and the molecular z-axis for convenience initially.
c             And then it is transformed to B0 is necessary.
c
c   *Note that the sign of these relaxation constants are reversed
c    with respect to the time domain formalism of the relaxation processes     
c    since we are calculating the frequency spectrum directly.
c
c      Original program by G. Moro with modifications by D. Budil
c      Further modification by Sanghyuk Lee to calculate the slow-motional
c              2D-FT spectra.
c
c       flags
c       -----
c
c             fld   : True if |lr-lc| <=2.
c                     Used as flag for bandwidth of Liouville operator
c                     matrix elements. The limit on the L-bandwidth is due
c                     to the restriction of spin operator tensors to
c                     ranks 0 and 2.
c
c             fldmd : True if |lr-lc| <=2 or mr=mc, false otherwise.
c                     Used as flag for possible non-zero matrix 
c                     elements. The L-bandwidth restriction is 
c                     explained above and the diffusion operator 
c                     matrix elements can be found only when mr=mc.
c
c             flk0  : True if lr=lc and kr=kc.
c                     Used as a flag for determing the existence of
c                     non-zero Liouville (A-tensor) and diffusion
c                     (Heisenberg spin exchange) matrix elements.
c
c	      fkm0  : True if kr=kc and mr=mc
c	              
c       Includes :
c               stddim.inc
c               rndoff.inc
c               eprdat.inc
c               eprmat.inc
c               maxl.inc
c               physcn.inc
c
c       Uses:
c               cd2km.f
c               anxlk.f
c               ipar.f
c               w3j.f
c
c----------------------------------------------------------------------
      subroutine matrxo(ierr)
c
      implicit none
c
      include 'stddim.inc'
      include 'rndoff.inc'
      include 'eprdat.inc'
      include 'indexf.inc'
      include 'eprmat.inc'
      include 'maxl.inc'
      include 'physcn.inc'
c
      logical fnz,fld,flk0,diftrm,sumtrm,fldmd
      logical newlr,newjkr,newkr,newjmr,newmr,newpnr,
     #        newlc,newjkc,newkc,newjmc,newmc,newpnc
      integer ioldlr,ioldjkr,ioldkr,ioldjmr,ioldmr,ioldpnr,
     #        ioldlc,ioldjkc,ioldkc,ioldjmc,ioldmc,ioldpnc
c
      double precision c,c1,c2,c3,c4,cdfd,cdfd1,cdfd2,
     #       cdff,cdffu,cga0,cga2,cgam,
     #       cliou,cliou0,cliou1,cliou2,clioua,clioua1,
     #       clioua2,clioug,clioug1,clioug2,
     #       cnorm,cnk,cnl,cnp,cpmkr,cplkc,cplmc,cnpr,cnpc,ct,
     #       d1,d2,dsq2,dsq12,dsq18,dsq23,
     #       fii,fki,ga,gg,ossc,pmxk,pmxl,pmz,
     #       ra,ra1,ra2,rg,rg1,rg2,rpl,rmi,rz,rj,rp,
     #       sa1,sg1,wliou1,wliou2,z,
     #       t2ed,t2ef,ct2efi
c
      double precision zero,one
      parameter (zero=0.0d0,one=1.0d0)
c
      double precision d2km(2,5,5)
      integer i,j,l,li,ierr,iper,
     #       ipnr,ipnc,ipnd,ipns,ipndab,ipnsab,
     #       iqnr,iqnc,iqnd,iqndab,
     #       lr,lc,ld,ldabs,lrprod,jkr,jkc,jkd,jmr,jmc,jmd,
     #       kr,kc,kd,ks,kdabs,ksabs,kdi,ksi,kip,
     #       mr,mc,md,ms,mdabs,msabs,
     #       nrow,ncol,nel,nelr,neli
c
      integer ipar
      double precision w3j
      external ipar,w3j
c
c######################################################################
c
      if (ndim.gt.mxdim) then
        ierr=2
        go to 9999
      end if
c
      if(abs(zeen).gt.rndoff) fnz=.true.
c
c----------------------------------------------------------------------
c     define constants
c----------------------------------------------------------------------
c
      dsq2=dsqrt(2.0D0)
      dsq12=one/dsq2
      dsq18=one/dsqrt(8.0D0)
      dsq23=dsqrt(2.0D0/3.0D0)
c
c----------------------------------------------------------------------
c     Scale diffusion constants
c----------------------------------------------------------------------
c
      ct=g0*betae/hbar
      rpl=0.5d0*(dx+dy)/ct
      rmi=0.25d0*(dx-dy)/ct
      rz=dz/ct
      rj=djf/ct
      rp=djfprp/ct
      ossc=oss/ct
c        
      t2ed=t2edi/ct
      t2ef=t2efi/ct
c
      fii=0.25d0*dble(in2*(in2+2))
c
c    Non-Brownian motional parameters
c
      if(ipdf.eq.1) then
        pmxl=dx*tl
        pmz=dz*tkzz
        pmxk=dx*tkxy
      else
        pmxl=zero
        pmz=zero
        pmxk=zero
      end if
c
c   Correction term for anisotropic viscosity
c
      if((ipdf.eq.2).and.(ist.eq.0)) then
        rpl=rpl+rp
        rz=rz+rp
      end if
c
c----------------------------------------------------------------------
c                                 2
c     Get Wigner rotation matrix D  (0,psi,0) for director tilt.
c                                 KM
c
c     Note: resultant matrix will be real-valued and d2km(2,i,j)=zero
c
c                              2
c           d2km(1,i+3,j+3) = d  (psi)
c                              ij
c----------------------------------------------------------------------
c
      call cd2km(d2km,zero,psi,zero)
c
c----------------------------------------------------------------------
c                               L
c     Calculate the quantities X  used in the potential-dependent
c                               K
c     portion of the diffusion operator
c----------------------------------------------------------------------
c
      call anxlk(rpl,rmi,rz)
c
c----------------------------------------------------------------------
c     Initialize counters
c----------------------------------------------------------------------
c
      nelr=0
      neli=0
      nel=0
c
      iper=ips
      ioldlr=-1
      ioldjkr=0
      ioldkr=1000
      ioldjmr=0
      ioldmr=1000
      ioldpnr=-in2-1
c
c----------------------------------------------------------------------
c     **** Loop over rows ***
c----------------------------------------------------------------------
c
      do 200 nrow=1,ndim
        lr=l1(nrow)
        jkr=jk1(nrow)
        kr=k1(nrow)
        jmr=jm1(nrow)
        mr=m1(nrow)
        ipnr=pi1(nrow)
        iqnr=qi1(nrow)
c
        newlr=lr.ne.ioldlr
        newjkr=newlr.or.(jkr.ne.ioldjkr)
        newkr=newjkr.or.(kr.ne.ioldkr)
        newjmr=newkr.or.(jmr.ne.ioldjmr)
        newmr=newjmr.or.(mr.ne.ioldmr)
        newpnr=newmr.or.(ipnr.ne.ioldpnr)
c
        if(newlr) then
          ioldlr=lr
          lrprod=lr*(lr+1)
        end if
c
        if(newjkr) ioldjkr=jkr
c
        if(newkr) then
          ioldkr=kr
c
c         ** Calculate isotropic part of diffusion operator
c
          c1=dble(lrprod)
          if(ipdf.ne.0) then
            c2=one+pmxl*c1
            if(dabs(pl).gt.rndoff) c2=c2**pl
            c1=c1/c2
          end if
          cdfd=rpl*c1
c
          c1=dble(kr*kr)
          c2=rz
          c3=rpl
          if(ipdf.ne.0) then
            c4=one+pmz*c1
            if(dabs(pkzz).gt.rndoff) c4=c4**pkzz
            c2=c2/c4
            c4=one+pmxk*c1
            if(dabs(pkxy).gt.rndoff) c4=c4**pkxy
            c3=c3/c4
          end if
          cdfd=cdfd+c1*(c2-c3)
c
c         ** Discrete jump motion
c
          if(ist.ne.0) then
            i=kr/ist
            if((i*ist).ne.kr) cdfd=cdfd+rj
          end if
c
c         Rotationally invariant line width : T2 electronic relaxation
c
          if(iper.ne.0) cdfd=cdfd+t2ed
c
          cdfd1=cdfd
c
        end if
c
        if(newjmr) ioldjmr=jmr
c
        if(newmr) then
          ioldmr=mr
          cpmkr=ipar(mr+kr)
c
c   Anisotropic viscosity term in diffusion operator
c
          if(ipdf.eq.2) cdfd=cdfd1+dble(mr*mr)*(rj-rp)
c
        end if
c
        if(newpnr) then
          ioldpnr=ipnr
          if((ipnr.eq.0).and.(mr.eq.0)) then
            cnpr=dsq2
          else
            cnpr=1.0D0
          end if
        end if
c
        jzmat(nrow)=neli+1
        kzmat(nrow)=nelr+1
c
c----------------------------------------------------------------------
c   **** Loop over columns (note only upper diagonal) ****
c----------------------------------------------------------------------
c
        ioldlc=-1
        ioldjkc=0
        ioldkc=1000
        ioldjmc=0
        ioldmc=1000
        ioldpnc=-in2-1
c
        do 300 ncol=nrow,ndim
          lc=l1(ncol)
          jkc=jk1(ncol)
          kc=k1(ncol)
          jmc=jm1(ncol)
          mc=m1(ncol)
          ipnc=pi1(ncol)
          iqnc=qi1(ncol)
c
          newlc=lc.ne.ioldlc
          newjkc=newlc.or.(jkc.ne.ioldjkc)
          newkc=newjkc.or.(kc.ne.ioldkc)
          newjmc=newkc.or.(jmc.ne.ioldjmc)
          newmc=newjmc.or.(mc.ne.ioldmc)
          newpnc=newmc.or.(ipnc.ne.ioldpnc)
c
          if(newlc) then
            ioldlc=lc
            ld=lr-lc
            ldabs=iabs(ld)
            fld=ldabs.le.2
            cnl=dsqrt((2.D0*lr+1.D0)*(2.D0*lc+1.D0))
c
c           Angular variation of the line width : T2 electronic relaxation
c
            ct2efi=w3j(lr,2,lc,mr,0,-mr)*w3j(lr,2,lc,kr,0,-kr)
            ct2efi=cnl*cpmkr*t2ef*ct2efi*d2km(1,3,3)
c
          end if
c
          if(newjkc) then
            ioldjkc=jkc
            jkd=jkr-jkc
          end if
c
          if(newkc) then
            ioldkc=kc
            kd=kr-kc
            ks=kr+kc
            kdabs=iabs(kd)
            ksabs=iabs(ks)
            cplkc=ipar(lc+kc)
c
            flk0=(ld.eq.0).and.(kd.eq.0)
c
            if((kr.eq.0).and.(kc.eq.0)) then
              cnk=0.5D0
            else if((kr.ne.0).and.(kc.ne.0)) then
              cnk=one
            else
              cnk=dsq12
            end if
c
c---------------------------------------------------------------------
c       Calculate R(mu,l) as in Meirovitch, Igner, Igner, Moro, & Freed
c       J. Phys. Chem. 77 (1982) p. 3935, Eqs. A42 & A44
c---------------------------------------------------------------------
c
            ra=zero
            rg=zero
            if(fld) then
              ra1=zero
              rg1=zero
              if(kdabs.le.2) then
                if(jkd.eq.0) then
                  ga=fad(1,kd+3)
                  gg=fgd(1,kd+3)
                else
                  ga=fad(2,kd+3)*jkr
                  gg=fgd(2,kd+3)*jkr
                end if
                z=w3j(lr,2,lc,kr,-kd,-kc)
                ra1=ga*z
                rg1=gg*z
              end if
c
              ra2=zero
	      rg2=zero
              if(ksabs.le.2) then
                if(jkd.eq.0) then
                  ga=fad(1,ks+3)
                  gg=fgd(1,ks+3)
                else
                  ga=fad(2,ks+3)*jkr
                  gg=fgd(2,ks+3)*jkr
                end if
                z=w3j(lr,2,lc,kr,-ks,kc)
                ra2=ga*z
                rg2=gg*z
              end if
c                                      --- for g tensor
              rg=rg1+cplkc*jkc*rg2

c                                      --- for A tensor
              if(in2.ne.0) ra=ra1+cplkc*jkc*ra2
	    end if
c
c----------------------------------------------------------------------
c       Calculate off-diagonal terms of the diffusion operator,
c       including the potential-dependent portion and the rhombic
c       component of the isotropic diffusion operator.
c
c       See Meirovitch, Igner, Igner, Moro,and Freed,
c       J. Chem. Phys. 77 (1982) pp.3933-3934, and especially
c       Eqns. A22-24 and A40 for more information.
c----------------------------------------------------------------------
c
c           -------- Rhombic part of isotropic diffusion operator
c
            if((ld.eq.0).and.(kr+2.eq.kc)) then
              cdff=rmi*dsqrt(dble((lr+kc-1)*(lr+kc)*
     #             (lr-kc+1)*(lr-kc+2)))/cnk
            else
              cdff=zero
            end if
c
c           -------- Potential-dependent part of diffusion operator
c
            if((ipt.ne.0).and.(ldabs.le.lband).and.(ipar(ks).eq.1)
     #        .and.(jkd.eq.0).and.((kdabs.le.kband).or.
     #        (ksabs.le.kband))) then
c
              kdi=1+kdabs/2
              ksi=1+ksabs/2
              c1=cpmkr*cnl*cnk 
c
c             Loop over L and K indices in sum over terms in potential
c
              do 444 l=0,lband,2
                cdffu=0.0D0
                li=1+l/2
                if (ksi.le.li.and. abs(xlk(li,ksi)).ge.rndoff)
     #               cdffu=cplkc*jkc*xlk(li,ksi)*w3j(lr,l,lc,kr,-ks,kc)
                if (kdi.le.li.and. abs(xlk(li,kdi)).ge.rndoff)
     #               cdffu=cdffu+xlk(li,kdi)*w3j(lr,l,lc,kr,-kd,-kc)
c
                if (abs(cdffu).gt.rndoff) cdff=cdff+
     #                           w3j(lr,l,lc,mr,0,-mr)*c1*cdffu
 444          continue
c
            end if
          end if
c
          if(newjmc) then
            ioldjmc=jmc
            jmd=jmr-jmc
          end if
c
          if(newmc) then
            ioldmc=mc
            md=mr-mc
            ms=mr+mc
            mdabs=iabs(md)
            msabs=iabs(ms)
            cplmc=ipar(lc+mc)
c
c----------------------------------------------------------------------
c     set flags and zero out matrix elements
c     if no non-zero elements are possible
c----------------------------------------------------------------------
c
            if(fld.or.(md.eq.0)) then
              fldmd=.true.
            else
              cliou=0.0D0
              cgam=0.0D0
              fldmd=.false.
            end if
c
c----------------------------------------------------------------------
c     get appropriate 3-J symbols if non-zero Liouville
c     operator matrix elements are possible
c----------------------------------------------------------------------
c
            wliou1=zero
            wliou2=zero
            if(fld) then
              wliou1=w3j(lr,2,lc,mr,-md,-mc)
              wliou2=w3j(lr,2,lc,mr,-ms,mc)
            end if
c
          end if
c
          if(newpnc) then
            ioldpnc=ipnc
            ipnd=ipnr-ipnc
            ipndab=iabs(ipnd)
            ipns=ipnr+ipnc
            ipnsab=iabs(ipns)
c
            if((ipnc.eq.0).and.(mc.eq.0)) then
              cnpc=dsq2
            else
              cnpc=1.0D0
            end if
c
            cnp=1.0D0/(cnpr*cnpc)
            cnorm=cnl*cnk*cnp*cpmkr
c
c----------------------------------------------------------------------
c     get director tilt dependence of Liouville
c     operator matrix elements
c----------------------------------------------------------------------
c
            d1=zero
            d2=zero
            if((ipndab.le.2).and.(mdabs.le.2))
     #                           d1=d2km(1,ipnd+3,md+3)*wliou1
c
            if((ipnsab.le.2).and.(msabs.le.2))
     #                           d2=d2km(1,ipns+3,ms+3)*wliou2
c
          end if
c
          if(fldmd) then
            iqnd=iqnr-iqnc
            iqndab=iabs(iqnd)
c
c----------------------------------------------------------------------
c    Liouville operator
c
c    Note: see Meirovitch, Igner,Igner,Moro, and Freed
c    J. Chem. Phys. 77 (1982) Appendix B p.3936-3937 for details.
c
c          Sa and Sg in this program also contain the
c          appropriate Clebsch-Gordon coefficient, and fki is
c          Ki in text.
c
c          The manner in which the isotropic contributions from the
c          spin Hamiltonian are included in this program is not clear
c          from the text, and  deserves some comments.
c          When Lc=Lr, both the (script) l=0 and l=2 ISTO's must
c          be included, whereas only l=2 terms appear off the diagonal.
c          The Clebsch-Gordan coefficients for these terms are
c          c(1,0,1,0;0,0) and c(1,+/-1,1,-/+1;0,0), or -/+ 1/sqrt(3),
c          respectively. Here, cga0 is set to +/- 1 instead of -/+
c          1/sqrt(3) because the extra factors are already included in g0
c          and a0 (note sign change). The isotropic terms are not
c          normalized by any factor.
c	   The cnk factor is not needed because the l=0 ISTO components
c	   have not been transformed by the K-symmetrization.
c          The factor cnl*cpmkr is canceled by the product of
c          w3j(lr,mr,0,0,lr,-mr) and w3j(lr,kr,0,0,lr,-kr), which
c          therefore do not need to be calculated.
c
c----------------------------------------------------------------------
c
            cliou=zero
c
            if(fld) then
              clioua=zero
              clioug=zero
              cliou0=zero
              cliou1=zero
              cliou2=zero
c
c        **   Hyperfine interaction   **
c
              if((jmd.eq.0).and.(in2.ne.0)) then
c                                         *** difference term ***
                clioua1=zero
                cliou1=zero
                if((ipndab.eq.iqndab).and.(ipndab.le.1).and.(mdabs.
     #            le.2))  then
c
                  cga0=zero
                  cga2=zero
                  if(ipnd.ne.0) then
c                                         *** pseudosecular term ***
c					      (delta pI .ne. 0)
                    kip=iqnr*iqnd+ipnr*ipnd
                    kip=kip*(kip-2)
                    fki=dsqrt(fii-0.25D0*kip)
                    sa1=-iper*ipnd*fki*dsq18
                    cga2=dsq12
                  else
c                                         *** secular term ***
c					      (delta pS,pI .eq. 0)
                    sa1=iper*iqnr*0.5D0
	            cga0=one
	            cga2=dsq23
                  end if
                  clioua1=sa1*d1*ra*cga2
                  if(flk0.and.(jkd.eq.0).and.(md.eq.0))
     #                                      cliou1=sa1*cga0*a0
                end if
c
c                                         *** sum term ***
                clioua2=zero
                cliou2=zero
                if((ipnsab.eq.iqndab).and.(ipnsab.le.1).and.(msabs.
     #            le.2))  then
c
                  cga0=zero
                  cga2=zero
                  if(ipns.ne.0) then
c                                         *** pseudosecular term ***
c					      (delta pI .ne. 0)
                    kip=iqnr*iqnd+ipnr*ipns
                    kip=kip*(kip-2)
                    fki=dsqrt(fii-0.25D0*kip)
                    sa1=-iper*ipns*fki*dsq18
                    cga2=dsq12
                  else
c                                         *** secular term ***
c					      (delta pS,pI .eq. 0)
                    sa1=iper*iqnr*0.5D0
	            cga0=one
	            cga2=dsq23
                  end if
c
                  clioua2=sa1*d2*ra*cga2
                  if(flk0.and.(jkd.eq.0).and.(ms.eq.0))
     #                                      cliou2=sa1*cga0*a0
                end if
c
                clioua=clioua1+jmc*cplmc*clioua2
                cliou0=cliou1+cliou2
c
              end if
c
c        **   Electronic Zeeman interaction   **
c
              cliou1=zero
              cliou2=zero
              clioug1=zero
              clioug2=zero
              if((jmd.eq.0).and.(iqnd.eq.0)) then
c
                if(flk0.and.(jkd.eq.0).and.(md.eq.0).and.
     #             (ipnd.eq.0))            cliou1=iper*b0
c
                if((ipnd.eq.0).and.(abs(rg).gt.rndoff)) then
                  sg1=iper*dsq23
                  clioug1=sg1*d1*rg
                end if
c
                if(flk0.and.(jkd.eq.0).and.(ms.eq.0).and.
     #             (ipns.eq.0))            cliou2=iper*b0
c
                if((ipns.eq.0).and.(abs(rg).gt.rndoff)) then
                  sg1=iper*dsq23
                  clioug2=sg1*d2*rg
                end if
c
                clioug=clioug1+jmc*cplmc*clioug2
                cliou0=(cliou0+cliou1+cliou2)*cnp
c
              end if
c
c        **   Nuclear Zeeman interaction   **
c
              cliou1=zero
              cliou2=zero
              if(fnz.and.(jmd.ne.0).and.flk0.and.(jkd.eq.0).
     #                           and.(iqnd.eq.0)) then
                if((ipnd.eq.0).and.(md.eq.0)) cliou1=ipnr*zeen
                if((ipns.eq.0).and.(ms.eq.0)) cliou2=ipnr*zeen
                cliou0=cliou0+cnp*(cliou1+cliou2)
              end if
c
              cliou=cnorm*(clioua+clioug)+cliou0
c
            end if
c
c----------------------------------------------------------------------
c           Diffusion operator
c----------------------------------------------------------------------
c
            cgam=zero
c
c      **   Heisenberg exchange   **
c
            if((dabs(oss).gt.rndoff).and.(ipnd.eq.0).and.flk0
     #         .and.(md.eq.0).and.(jkd.eq.0).and.(jmd.eq.0)) then
              c=zero
              if(iqnd.eq.0) c=one
              if((ipnr.eq.0).and.(lr.eq.0)) c=c-one/dble(in2+1)
              cgam=c*ossc
            end if
c
c      **   Potential-dependent terms   **
c
            if((ipnd.eq.0).and.(iqnd.eq.0).and.(md.eq.0).and.
     #        (jkd.eq.0).and.(jmd.eq.0)) then
              cgam=cgam+cdff
              if(fld.and.(kd.eq.0))   cgam=cgam+ct2efi
            end if
c
          end if
c
c----------------------------------------------------------------------
c         Store matrix elements
c----------------------------------------------------------------------
c
          if(nrow.eq.ncol) then
            cgam=cgam+cdfd
            zdiag(1,nrow)=cgam
            zdiag(2,nrow)=-cliou
          else
c
            if(abs(cliou).gt.rndoff) then
              nel=nel+1
              if(nel.gt.mxel) then
                ierr=1
                return
              else
                neli=neli+1
                zmat(neli)=-cliou
                izmat(neli)=ncol
              end if
            end if
c
            if(abs(cgam).gt.rndoff) then
              nel=nel+1
              if(nel.gt.mxel) then
                ierr=1
                return
              else
                nelr=nelr+1
                zmat(mxel-nelr+1)=cgam
                izmat(mxel-nelr+1)=ncol
              end if
            end if
          end if
c
c----------------------------------------------------------------------
c     Increment column counter
c----------------------------------------------------------------------
c
c  ****test*****
c         if (nrow.eq.1) then
c         if (ncol.eq.1)       write(6,332)
c         write(6,333) ncol,lc,jkc,kc,mc,ipnc,iqnc
c 332     format(2x,' ncol  lc    jkc   mc   ipnc  iqnc')
c 333     format(2x,7i6)
c         end if
c
          if(ncol.gt.mxdim) then
            ierr=2
            return
          end if
c----------------------------------------------------------------------
c         end loop over columns
c----------------------------------------------------------------------
 300    continue       
c----------------------------------------------------------------------
c       end loop over rows
c----------------------------------------------------------------------
 200  continue
c
      ierr=0
      neltot=nel
      nelre=nelr
      nelim=neli
c
      jzmat(ndim+1)=neli+1
      kzmat(ndim+1)=nelr+1
c
 9999 return
      end
