c  VERSION 1.0  (NLSPR version)   4/7/93
c*********************************************************************
c
c                       ==================
c                       SUBROUTINE : EVESA
c                       ==================
c
c     Subroutine version of EPRESA program. (intended for NLSP or MOMD)
c
c     It calculates the eigenvalues and eigenvectors for the general 
c     complex matrix using Rutishauser Algorithm, described by Gordon
c     and Messenger in "Electron-Spin Relaxation in Liquids", Muus and
c     Atkins, (1972).
c
c     On Entry :
c
c        npt1  : # of t1 values to be used in the determination of
c                weighting factors of diagonal eigenvectors
c        Parameters are passed through the common block /eprprm/.
c        Pulse propagator is passed through the common block /indexf/.
c        Off-diagonal starting vector is passed through the common
c                                                      block /indexf/.
c
c     On Exit  :
c
c        nev   : number of important eigenvalues
c        eval  : important eigenvalues
c        evec  : important eigenvectors
c        ierr  : error return (Rutishauser not converged.)
c
c     Includes :
c               nlsdim.inc
c               stdio.inc
c               eprprm.inc
c               parcom.inc
c               indexf.inc
c               stvcom.inc
c               eprmat.inc
c               rndoff.inc
c               wkspcm.inc
c
c     Uses :
c               matrxo.f
c               matrxd.f
c               csqz.f
c               cqrt.f
c               indexx.f
c
c*********************************************************************
c
      subroutine evesa(psval,stv,nev,eval,evec,npt1,ierr)
c
      implicit none
c
      include 'nlsdim.inc'
      include 'stdio.inc'
      include 'eprprm.inc'
      include 'parcom.inc'
      include 'indexf.inc'
      include 'stvcom.inc'
      include 'eprmat.inc'
      include 'rndoff.inc'
      include 'wkspcm.inc'
c
      integer i,j,k,icol,psval,ndim,bdw,nslow,nev,ierr,mxit,npt1
      double precision zr,zi,zero,sqtol,tol
      complex*16 czero,cone,eval,evec,wt,stv
      parameter (zero=0.D0,czero=(0.D0,0.D0),cone=(1.D0,0.D0))
      dimension eval(mxdim),evec(mxdim,mxdim),stv(mxdim,mxnpt)
c
c#####################################################################
c
c---------------------------------------------------------------------
c     calculate the matrix elements
c---------------------------------------------------------------------
c
      ierr=0
      if (psval.ne.0) then
         ndim=ndimo
         call matrxo(bdw)
         if (idebug.ne.0) then
c                                ** store matrix elements **
            open (unit=ludisk,file='matrx.mtxx',status='unknown',
     #           access='sequential',form='formatted')
            do 20 i=1,ndim
               write (ludisk,1050) i,l1(i),jk1(i),k1(i),jm1(i),m1(i),
     #                     pi1(i),qi1(i)
               if (cdabs(cmat(i,1)).gt.rndoff)
     #              write (ludisk,1060) i,cmat(i,1),
     #                 l1(i),jk1(i),k1(i),jm1(i),m1(i),pi1(i),qi1(i)
               icol=ndim-i+1
               do 30 j=2,icol
                  if (cdabs(cmat(i,j)).gt.rndoff) then
                     k=i+j-1
                     write (ludisk,1060) k,cmat(i,j),l1(k),jk1(k),
     #                    k1(k),jm1(k),m1(k),pi1(k),qi1(k)
                  end if
 30            continue
 20         continue
            close (unit=ludisk)
         end if
      else
         ndim=ndimd
         call matrxd(bdw)
         if (idebug.ne.0) then
c                                ** store matrix elements **
            open (unit=ludisk,file='matrx.mtxz',status='unknown',
     #           access='sequential',form='formatted')
            do 40 i=1,ndim
               write (ludisk,1050) i,dl1(i),djk1(i),dk1(i),djm1(i),
     #                     dm1(i),dpi1(i),dqi1(i)
               if (cdabs(cmat(1,i)).gt.rndoff)
     #              write (ludisk,1060) i,cmat(1,i),dl1(i),djk1(i),
     #                 dk1(i),djm1(i),dm1(i),dpi1(i),dqi1(i)
               icol=ndim-i+1
               do 50 j=2,icol
                  if (cdabs(cmat(j,i)).gt.rndoff) then
                     k=i+j-1
                     write (ludisk,1060) k,cmat(j,i),dl1(k),djk1(k),
     #                    dk1(k),djm1(k),dm1(k),dpi1(k),dqi1(k)
                  end if
 50            continue
 40         continue
            close (unit=ludisk)
         end if
      end if
c
c---------------------------------------------------------------------
c     convert the complex symmetric matrix into tridiagonal matrix
c---------------------------------------------------------------------
c
      sqtol=zero
c
      do 100 j=1,ndim
      do 100 i=1,ndim
 100     evec(i,j)=czero
c
      do 110 i=1,ndim
 110     evec(i,i)=cone
c
      call csqz(cmat,evec,mxdim,mxdim,mxdim,mxdim,ndim,bdw,sqtol)
c
c---------------------------------------------------------------------
c     Solve tridiagonal matrix for eigenvalues and eigenvectors
c---------------------------------------------------------------------
c
      mxit=100
c
      do 160 i=1,ndim
 160     cwsp1(i)=czero
c
      call cqrt(cmat(1,1),cmat(1,2),evec,ndim,mxdim,mxdim,
     #          mxit,tol,cwsp1,ierr)
c
      if (ierr.ne.0) return
c
c---------------------------------------------------------------------
c     keep the eigenvalues whose real part is smaller than 20.0 gauss
c     (fastly decaying eigenmodes do not contribute to the signal)
c---------------------------------------------------------------------
c
      nslow=0
      do 180 i=1,ndim
        if (dreal(cmat(i,1)).lt.2000.0d0) then
          nslow=nslow+1
          cmat(nslow,1)=cmat(i,1)
          do 185 j=1,ndim
 185        evec(j,nslow)=evec(j,i)
        end if
 180  continue
c
c---------------------------------------------------------------------
c     calculate weighting of each eigenvector on the starting vector
c     (This includes only the slowly decaying components.)
c---------------------------------------------------------------------
c
      nev=0
c
      if (psval.ne.0) then
c				*** off-diagonal space
        do 300 i=1,nslow
          wt=czero
          do 301 j=1,ndim
 301        wt=wt+evec(j,i)*stv(j,1)
          cwsp2(i)=wt*wt
          rwsp(i)=cdabs(cwsp2(i))
          if (rwsp(i).gt.wtol) nev=nev+1
 300    continue
      else
c				*** diagonal space
c     convert the eigenvectors of the diagonal space into the
c     corresponding off-diagonal space using the pulse propagator
c
        do 350 i=1,nslow
          k=1
          do 360 j=1,ndimo
            if (pid(j).eq.1) then
              evec(j,i)=-pp(k)*evec(k,i)
              k=k+1
            else
              evec(j,i)=-pp(k)*evec(k,i)-pp(k+1)*evec(k+1,i)
              k=k+2
            end if
 360      continue
c
          rwsp(i)=zero
          do 370 j=1,npt1
            wt=czero
            do 375 k=1,ndimo
 375          wt=wt+evec(k,i)*stv(k,j)
            wt=wt*wt
            if (cdabs(wt).gt.rwsp(i)) then
               rwsp(i)=cdabs(wt)
               cwsp2(i)=wt
            end if
 370      continue
          if (rwsp(i).gt.wtol) nev=nev+1
 350    continue
c
      end if
c
c---------------------------------------------------------------------
c     order the eigenvalues and eigenvectors in weighting factors
c---------------------------------------------------------------------
c
      if (nslow.gt.1) then
         call indexx(nslow,rwsp,iwsp)
      else
         iwsp(1)=1
      end if
c
      do 420 i=1,nev
        eval(i)=cmat(iwsp(i),1)
 420    cwsp2(i)=cwsp2(iwsp(i))
c
      do 430 i=1,ndimo
        do 440 j=1,nslow
 440      cwsp1(j)=evec(i,j)
        do 450 j=1,nev
 450      evec(i,j)=cwsp1(iwsp(j))
 430  continue
c
      if (idebug.ne.0) then
         if (ievec.eq.0) then
            write (ludeb,1020)
            write (ludeb,1025) (i,eval(i),cwsp2(i),i=1,nev)
         else
            do 460 i=1,nev
               write (ludeb,1030) i,eval(i),cwsp2(i)
               write (ludeb,1040) (j,evec(j,i),j=1,ndimo)
 460        continue
         end if
      end if
c
      return
c
c=====================================================================
c     format statements
c=====================================================================
c
 1020 format(/' ** Eigenvalues **'/)
 1025 format(2x,i3,2x,g14.7,1x,g14.7,2x,g14.7,1x,g14.7)
 1030 format(/,' ** Eigenvalue ',i3,' : ',g12.5,1x,g12.5,2x,g12.5,
     #       1x,g12.5,/)
 1040 format(5x,i3,2x,g14.7,1x,g14.7)
c
 1050 format (//,11x,'*** Row : ',i5,' ***',11x,'<',6(i3,','),i3,'|',
     #        /,3x,'Col',3x,'Real Part',6x,'Imaginary Part',6x,
     #        'L  jK   K  jqM  M  pI  qI',/)
 1060 format (1x,i5,' (',g14.7,',',g14.7,')   ',
     # '|',6(i3,','),i3,'>')
      end
